"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /**
                                                                                                                                                                                                                                                                               * Tests for DOM utility functions.
                                                                                                                                                                                                                                                                               * https://github.com/RobertoPrevato/KingTable
                                                                                                                                                                                                                                                                               *
                                                                                                                                                                                                                                                                               * Copyright 2018, Roberto Prevato
                                                                                                                                                                                                                                                                               * https://robertoprevato.github.io
                                                                                                                                                                                                                                                                               *
                                                                                                                                                                                                                                                                               * Licensed under the MIT license:
                                                                                                                                                                                                                                                                               * http://www.opensource.org/licenses/MIT
                                                                                                                                                                                                                                                                               */


var _dom = require("../scripts/dom");

var _dom2 = _interopRequireDefault(_dom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("DOM utilities", function () {

  afterAll(function () {
    // remove event handlers
    _dom2.default.offAll();
    // empty body
    _dom2.default.empty(document.body);
  });

  it("must distinguish DOM elements", function () {
    var a = document.createElement("script");
    expect(_dom2.default.isElement(a)).toEqual(true);

    expect(_dom2.default.isElement({})).toEqual(false);
  });

  it("must support events namespaces", function () {
    var name = "click.removable";

    var parts = _dom2.default.splitNamespace(name);
    expect(parts.length).toEqual(2);
    expect(parts[0]).toEqual("click");
    expect(parts[1]).toEqual("removable");

    var name = "click.removable.aaa";
    var parts = _dom2.default.splitNamespace(name);
    expect(parts.length).toEqual(2);
    expect(parts[0]).toEqual("click");
    expect(parts[1]).toEqual("removable.aaa");
  });

  it("must add and remove classes", function () {
    var a = document.createElement("p");
    _dom2.default.addClass(a, "waa");

    expect(a.classList.contains("waa")).toEqual(true);

    _dom2.default.addClass(a, "aaa bbb");

    expect(a.classList.contains("aaa")).toEqual(true);
    expect(a.classList.contains("bbb")).toEqual(true);

    _dom2.default.removeClass(a, "waa");
    expect(a.classList.contains("waa")).toEqual(false);
  });

  it("must check if elements have classes", function () {
    var a = document.createElement("p");
    a.classList.add("aa");

    expect(_dom2.default.hasClass(a, "aa")).toEqual(true);
    expect(_dom2.default.hasClass(a, "bb")).toEqual(false);
  });

  it("must identify password elements", function () {
    var a = document.createElement("input");
    a.type = "text";
    expect(_dom2.default.isPassword(a)).toEqual(false);

    a.type = "password";
    expect(_dom2.default.isPassword(a)).toEqual(true);

    expect(_dom2.default.isPassword(null)).toEqual(null);
  });

  it("must allow to fire events on elements", function () {
    var a = document.createElement("span");

    document.body.appendChild(a);
    var k = 0;
    _dom2.default.on(a, "click", function (e) {
      k++;
    });

    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(1);

    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(2);
  });

  it("must allow to define multiple events handlers on elements", function () {
    var a = document.createElement("span");

    document.body.appendChild(a);
    var k = 0,
        j = 0;
    _dom2.default.on(a, "click", function (e) {
      k++;
    });

    _dom2.default.on(a, "click", function (e) {
      j++;
    });
    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(1);
    expect(j).toEqual(1);

    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(2);
    expect(j).toEqual(2);
  });

  it("must allow to unset event handlers by namespace", function () {
    var a = document.createElement("span");

    document.body.appendChild(a);
    var k = 0,
        j = 0;
    _dom2.default.on(a, "click.foo", function (e) {
      k++;
    });

    _dom2.default.on(a, "click.ofo", function (e) {
      j++;
    });
    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(1);
    expect(j).toEqual(1);

    // off events with `ofo` namespace:
    _dom2.default.off(a, ".ofo");

    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(2, "event handler for k must still be active");
    expect(j).toEqual(1, "event handler for j must be removed by namespace");
  });

  it("must allow to fire and handle events, with namespace", function () {
    var a = document.createElement("span");

    document.body.appendChild(a);
    var k = 0;
    _dom2.default.on(a, "click.foo", function (e) {
      k++;
    });

    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(1);
  });

  it("must allow to fire events on elements, with extra data", function () {
    var a = document.createElement("span");

    document.body.appendChild(a);
    var k = 0;
    _dom2.default.on(a, "click", function (e, data) {
      k++;
      expect(typeof data === "undefined" ? "undefined" : _typeof(data)).toEqual("object", "extra data must be an object");
      expect(data.test).toEqual(true, "extra data must match the item passed to fire function");
    });

    // fire event
    _dom2.default.fire(a, "click", { test: true });
    expect(k).toEqual(1);

    // fire event
    _dom2.default.fire(a, "click", { test: true });
    expect(k).toEqual(2);
  });

  it("must allow to fire events on elements, with extra data", function () {
    var a = document.createElement("span");

    document.body.appendChild(a);
    var k = 0;
    _dom2.default.on(a, "click", function (e, data) {
      k++;
      expect(typeof data === "undefined" ? "undefined" : _typeof(data)).toEqual("object", "extra data must be an object");
      expect(data.test).toEqual(true, "extra data must match the item passed to fire function");
    });

    // fire event
    _dom2.default.fire(a, "click", { test: true });
    expect(k).toEqual(1);

    // fire event
    _dom2.default.fire(a, "click", { test: true });
    expect(k).toEqual(2);
  });

  it("must allow to unset all event handlers at once", function () {
    var a = document.createElement("span");

    document.body.appendChild(a);
    var k = 0,
        j = 0;
    _dom2.default.on(a, "click.foo", function (e) {
      k++;
    });

    _dom2.default.on(a, "click.ofo", function (e) {
      j++;
    });
    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(1);
    expect(j).toEqual(1);

    // off events with `ofo` namespace:
    _dom2.default.offAll();

    // fire event
    _dom2.default.fire(a, "click");
    expect(k).toEqual(1, "event handler for k must be removed");
    expect(j).toEqual(1, "event handler for j must be removed");
  });

  it("must allow to get the siblings of an element", function () {
    var a = document.createElement("ul");
    a.innerHTML = "\n    <li>lorem</li>\n    <li>ipsum</li>\n    <li>dolor</li>\n    <li>sit</li>\n    ";

    var ipsum = a.children[1];
    expect(ipsum.innerText).toEqual("ipsum");

    var siblings = _dom2.default.siblings(ipsum);
    expect(siblings instanceof Array).toEqual(true, "siblings return object must be an array");
    expect(siblings.length).toEqual(3, "ipsum element has three siblings");
    expect(siblings[0].innerText).toEqual("lorem");
    expect(siblings[1].innerText).toEqual("dolor");
    expect(siblings[2].innerText).toEqual("sit");
  });

  it("must allow to get the next siblings of an element", function () {
    var a = document.createElement("ul");
    a.innerHTML = "\n    <li>lorem</li>\n    <li>ipsum</li>\n    <li>dolor</li>\n    <li>sit</li>\n    ";

    var ipsum = a.children[1];
    expect(ipsum.innerText).toEqual("ipsum");

    var siblings = _dom2.default.nextSiblings(ipsum);
    expect(siblings instanceof Array).toEqual(true, "siblings return object must be an array");
    expect(siblings.length).toEqual(2, "ipsum element has two next siblings");
    expect(siblings[0].innerText).toEqual("dolor");
    expect(siblings[1].innerText).toEqual("sit");
  });

  it("must allow to get the previous siblings of an element", function () {
    var a = document.createElement("ul");
    a.innerHTML = "\n    <li>lorem</li>\n    <li>ipsum</li>\n    <li>dolor</li>\n    <li>sit</li>\n    ";

    var ipsum = a.children[1];
    expect(ipsum.innerText).toEqual("ipsum");

    var siblings = _dom2.default.prevSiblings(ipsum);
    expect(siblings instanceof Array).toEqual(true, "siblings return object must be an array");
    expect(siblings.length).toEqual(1, "ipsum element has one previous sibling");
    expect(siblings[0].innerText).toEqual("lorem");
  });

  it("must allow to get the closest element by tagname", function () {
    var a = document.createElement("ul");
    a.innerHTML = "\n    <li>lorem\n      <div>\n        <div>\n          <span class=\"foo\">ipsum</span>\n        </div>\n      </div>\n    </li>\n    ";

    var lorem = a.querySelector(".foo");
    expect(lorem.innerText).toEqual("ipsum");

    var closestDiv = _dom2.default.closestWithTag(lorem, "div");
    expect(closestDiv).toEqual(lorem.parentElement, "closest div of .foo is its parent");

    var closestLi = _dom2.default.closestWithTag(lorem, "li");
    expect(closestLi).toEqual(a.children[0], "closest li of .foo is list's first child");

    var closestUl = _dom2.default.closestWithTag(lorem, "ul");
    expect(closestUl).toEqual(a, "closest ul of .foo is list's element");
  });

  it("must return the element itself as its closest element by tag", function () {
    var a = document.createElement("ul");
    a.innerHTML = "\n    <li>lorem\n      <div>\n        <div>\n          <span class=\"foo\">ipsum</span>\n        </div>\n      </div>\n    </li>\n    ";

    var lorem = a.querySelector(".foo");
    expect(lorem.innerText).toEqual("ipsum");

    var closestSpan = _dom2.default.closestWithTag(lorem, "span");
    expect(closestSpan).toEqual(lorem, "closest span of .foo is itself");
  });

  it("must return the element itself as its closest element by tag, excluding itself", function () {
    var a = document.createElement("ul");
    a.innerHTML = "\n    <li>lorem\n      <div>\n        <div>\n          <span class=\"foo\">ipsum</span>\n        </div>\n      </div>\n    </li>\n    ";

    var lorem = a.querySelector(".foo");
    expect(lorem.innerText).toEqual("ipsum");

    var closestSpan = _dom2.default.closestWithTag(lorem, "span", true);
    expect(closestSpan).toEqual(undefined, "closest span of .foo, beside itself, is undefined");
  });
});