"use strict";

var _html = require("../scripts/data/html");

describe("Virtual element classes", function () {
  it("must allow to generate HTML for a single element", function () {
    var a = new _html.VHtmlElement("span");
    expect(a.toString()).toEqual("<span></span>");
  });

  it("must handle boolean attribute `checked`", function () {
    var a = new _html.VHtmlElement("input", {
      "type": "checkbox",
      "checked": true,
      "name": "foo"
    });
    expect(a.toString()).toEqual("<input type=\"checkbox\" checked name=\"foo\" />");
  });

  it("must handle boolean attribute `multiple`", function () {
    var a = new _html.VHtmlElement("select", {
      "multiple": true,
      "name": "foo"
    }, [new _html.VHtmlElement("option", {
      "value": 1
    }, new _html.VTextElement("A")), new _html.VHtmlElement("option", {
      "value": 2
    }, new _html.VTextElement("B")), new _html.VHtmlElement("option", {
      "value": 3
    }, new _html.VTextElement("C"))]);
    expect(a.toString(2, "_")).toEqual("<select multiple name=\"foo\">\n__<option value=\"1\">\n____A\n__</option>\n__<option value=\"2\">\n____B\n__</option>\n__<option value=\"3\">\n____C\n__</option>\n</select>\n");
  });

  it("must allow to generate HTML for a single element with text content", function () {
    var a = new _html.VHtmlElement("span", null, [new _html.VTextElement("Hello World!")]);
    expect(a.toString()).toEqual("<span>Hello World!</span>");
  });

  it("must allow to generate HTML with single children passed as third parameter", function () {
    var a = new _html.VHtmlElement("span", null, new _html.VTextElement("Hello World!"));
    expect(a.toString()).toEqual("<span>Hello World!</span>");
  });

  it("must escape a text element content", function () {
    var a = new _html.VHtmlElement("span", null, [new _html.VTextElement("<p>Hello World!</p>")]);
    expect(a.toString()).toEqual("<span>&lt;p&gt;Hello World!&lt;/p&gt;</span>");
  });

  it("must support null or undefined text content", function () {
    var a = new _html.VHtmlElement("span", null, [new _html.VTextElement(null)]);
    expect(a.toString()).toEqual("<span></span>");
  });

  it("must support a text element non-strings parameters", function () {
    var a = new _html.VHtmlElement("span", null, [new _html.VTextElement(200)]);
    expect(a.toString()).toEqual("<span>200</span>");
  });

  it("must allow to generate HTML for a single element with HTML comment", function () {
    var a = new _html.VHtmlElement("span", null, [new _html.VCommentElement("Hello World Comment!")]);
    expect(a.toString()).toEqual("<span><!--Hello World Comment!--></span>");
  });

  it("must remove inner comments tags from a comment virtual element", function () {
    var a = new _html.VHtmlElement("span", null, [new _html.VCommentElement("Hello <!--World--> Comment!")]);
    expect(a.toString()).toEqual("<span><!--Hello World Comment!--></span>");
  });

  it("must allow to generate HTML for a single element with attributes", function () {
    var a = new _html.VHtmlElement("span", {
      "id": "foo",
      "class": "beautiful-soup"
    });
    expect(a.toString()).toEqual("<span id=\"foo\" class=\"beautiful-soup\"></span>");
  });

  it("must allow to set id by attribute", function () {
    var a = new _html.VHtmlElement("span");
    a.id = "foo";
    expect(a.toString()).toEqual("<span id=\"foo\"></span>");
  });

  it("must recognize empty elements", function () {
    var a = new _html.VHtmlElement("img", {
      src: "/foo.jpg",
      alt: "Test image."
    });
    expect(a.empty).toEqual(true, "img is an empty element");
    expect(a.toString()).toEqual("<img src=\"/foo.jpg\" alt=\"Test image.\" />");
  });

  it("must recognize non-empty elements", function () {
    var a = new _html.VHtmlElement("div");

    expect(a.empty).toEqual(false, "div is an empty element");
    expect(a.toString()).toEqual("<div></div>");
  });

  it("must recognize empty elements", function () {
    var a = new _html.VHtmlElement("img", {
      src: "/foo.jpg",
      alt: "Test image."
    });

    expect(a.empty).toEqual(true, "img is an empty element");
    expect(a.toString()).toEqual("<img src=\"/foo.jpg\" alt=\"Test image.\" />");
  });

  it("must allow to define elements with children", function () {
    var a = new _html.VHtmlElement("div", {
      "class": "container"
    }, [new _html.VHtmlElement("span", { id: "foo" }), new _html.VHtmlElement("span", { id: "ufo" })]);

    expect(a.toString()).toEqual('<div class="container"><span id="foo"></span><span id="ufo"></span></div>');
  });

  it("must allow to write HTML with indentation", function () {
    var a = new _html.VHtmlElement("div", {
      "class": "container"
    }, [new _html.VHtmlElement("dl", null, [new _html.VHtmlElement("dt"), new _html.VHtmlElement("dd")])]);

    var expected = "<div class=\"container\">\n    <dl>\n        <dt></dt>\n        <dd></dd>\n    </dl>\n</div>\n".trim();
    expect(a.toString(4).trim()).toEqual(expected);
  });

  it("must allow to write HTML and text elements with indentation", function () {
    var a = new _html.VHtmlElement("div", {
      "class": "container"
    }, [new _html.VHtmlElement("dl", null, [new _html.VHtmlElement("dt", null, [new _html.VCommentElement("lorem ipsum dolor sit amet"), new _html.VTextElement("Name")]), new _html.VHtmlElement("dd", null, [new _html.VTextElement("Definition")])])]);

    var expected = "<div class=\"container\">\n    <dl>\n        <dt>\n            <!--lorem ipsum dolor sit amet-->\n            Name\n        </dt>\n        <dd>\n            Definition\n        </dd>\n    </dl>\n</div>\n".trim();
    expect(a.toString(4).trim()).toEqual(expected);
  });

  it("must allow to write HTML and text elements with indentation of desired size", function () {
    var a = new _html.VHtmlElement("div", {
      "class": "container"
    }, [new _html.VHtmlElement("dl", null, [new _html.VHtmlElement("dt", null, [new _html.VCommentElement("lorem ipsum dolor sit amet"), new _html.VTextElement("Name")]), new _html.VHtmlElement("dd", null, [new _html.VTextElement("Definition")])])]);

    var expected = "<div class=\"container\">\n  <dl>\n    <dt>\n      <!--lorem ipsum dolor sit amet-->\n      Name\n    </dt>\n    <dd>\n      Definition\n    </dd>\n  </dl>\n</div>\n".trim();
    expect(a.toString(2).trim()).toEqual(expected);
  });

  it("must allow to write HTML and text elements with indentation of desired size and char", function () {
    var a = new _html.VHtmlElement("div", {
      "class": "container"
    }, [new _html.VHtmlElement("dl", null, [new _html.VHtmlElement("dt", null, [new _html.VCommentElement("lorem ipsum dolor sit amet"), new _html.VTextElement("Name")]), new _html.VHtmlElement("dd", null, [new _html.VTextElement("Definition")])])]);

    var expected = "<div class=\"container\">\n__<dl>\n____<dt>\n______<!--lorem ipsum dolor sit amet-->\n______Name\n____</dt>\n____<dd>\n______Definition\n____</dd>\n__</dl>\n</div>\n".trim();
    expect(a.toString(2, "_").trim()).toEqual(expected);
  });

  it("must allow to mix HTML fragments to VHTML objects", function () {
    var a = new _html.VHtmlElement("div", {
      "class": "container"
    }, [new _html.VHtmlElement("p", null, ["<span>Hello</span><br>", "<span>World</span>"])]);

    var expected = "<div class=\"container\">\n__<p>\n__<span>Hello</span><br>\n__<span>World</span>\n__</p>\n</div>\n".trim();
    expect(a.toString(2, "_").trim()).toEqual(expected);
  });

  it("must generate HTML skipping hidden elements", function () {
    var a = new _html.VHtmlElement("div", {}, [new _html.VHtmlElement("p", null, new _html.VTextElement("Hello")), new _html.VHtmlElement("p", null, new _html.VTextElement("World")), new _html.VHtmlElement("p", null, new _html.VTextElement("Kitty"))]);
    a.children[2].hidden = true;
    expect(a.toString()).toEqual("<div><p>Hello</p><p>World</p></div>");
  });

  it("must allow to generate HTML elements without single root", function () {
    var a = new _html.VWrapperElement([new _html.VHtmlElement("p", null, new _html.VTextElement("Hello")), new _html.VHtmlElement("p", null, new _html.VTextElement("World"))]);
    expect(a.toString()).toEqual("<p>Hello</p><p>World</p>");
  });

  it("must allow to hide VWrapperElement", function () {
    var a = new _html.VHtmlElement("div", null, [new _html.VHtmlElement("p", null, new _html.VTextElement("Hello")), new _html.VHtmlElement("p", null, new _html.VTextElement("World")), new _html.VWrapperElement([new _html.VHtmlElement("span", null, new _html.VTextElement("Lorem ipsum")), new _html.VHtmlElement("span", null, new _html.VTextElement("dolor sit amet"))]), new _html.VHtmlElement("em", null, new _html.VTextElement("Kot"))]);
    expect(a.toString()).toEqual("<div><p>Hello</p><p>World</p><span>Lorem ipsum</span><span>dolor sit amet</span><em>Kot</em></div>");

    // hide
    a.children[2].hidden = true;
    expect(a.toString()).toEqual("<div><p>Hello</p><p>World</p><em>Kot</em></div>");
  });
}); /**
     * Tests for HTML generation classes.
     * https://github.com/RobertoPrevato/KingTable
     *
     * Copyright 2018, Roberto Prevato
     * https://robertoprevato.github.io
     *
     * Licensed under the MIT license:
     * http://www.opensource.org/licenses/MIT
     */