"use strict";

var _utils = require("../scripts/utils");

var _utils2 = _interopRequireDefault(_utils);

var _string = require("../scripts/components/string");

var _string2 = _interopRequireDefault(_string);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Tests for String utilities.
 * https://github.com/RobertoPrevato/KingTable
 *
 * Copyright 2018, Roberto Prevato
 * https://robertoprevato.github.io
 *
 * Licensed under the MIT license:
 * http://www.opensource.org/licenses/MIT
 */
describe("String utilities", function () {
  it("must allow to remove multiple spaces", function () {
    var a = "Hello   World";
    expect(_string2.default.removeMultipleSpaces(a)).toEqual("Hello World");
    a = "  Hello   World  ";
    expect(_string2.default.removeMultipleSpaces(a)).toEqual(" Hello World ");
  });

  it("must allow to convert strings to snake case", function () {
    // from mess to snake case:
    expect(_string2.default.snakeCase(" Hello World ")).toEqual("hello_world");

    // from mess to snake case:
    expect(_string2.default.snakeCase(" Hello    World ")).toEqual("hello_world");

    // from camel case to snake case:
    expect(_string2.default.snakeCase("helloWorld")).toEqual("hello_world");

    // from kebab case to snake case:
    expect(_string2.default.snakeCase(" hello-world  ")).toEqual("hello_world");

    // from mess to snake case:
    expect(_string2.default.snakeCase("_hello__world_")).toEqual("_hello_world_");
  });

  it("must allow to convert strings to camel case", function () {
    // from mess to snake case:
    expect(_string2.default.camelCase(" Hello World ")).toEqual("helloWorld");

    // from snake case to camel case:
    expect(_string2.default.camelCase("hello_world")).toEqual("helloWorld");

    // from snake case to camel case:
    expect(_string2.default.camelCase("   hello_  world")).toEqual("helloWorld");

    // from kebab case to camel case:
    expect(_string2.default.camelCase(" hello-world  ")).toEqual("helloWorld");
  });

  it("must allow to convert strings to kebab case", function () {
    expect(_string2.default.kebabCase(" Hello World ")).toEqual("hello-world");

    expect(_string2.default.kebabCase("hello_world")).toEqual("hello-world");

    expect(_string2.default.kebabCase("   hello_  world")).toEqual("hello-world");

    expect(_string2.default.kebabCase("helloWorld")).toEqual("hello-world");
  });

  it("must allow to center strings, in Python fashion", function () {
    expect(_string2.default.center("Hello", 10)).toEqual("  Hello   ");

    expect(_string2.default.center("Hello", 10, "*")).toEqual("**Hello***");

    expect(_string2.default.center("Hello", 11, "*")).toEqual("***Hello***");
  });

  it("must allow to left just strings, Python fashion", function () {
    expect(_string2.default.ljust("Hello", 10)).toEqual("Hello     ");

    expect(_string2.default.ljust("Hello", 10, "*")).toEqual("Hello*****");

    expect(_string2.default.ljust("Hello", 11, "*")).toEqual("Hello******");
  });

  it("must allow to right just strings, Python fashion", function () {
    expect(_string2.default.rjust("Hello", 10)).toEqual("     Hello");

    expect(_string2.default.rjust("Hello", 10, "*")).toEqual("*****Hello");

    expect(_string2.default.rjust("Hello", 11, "*")).toEqual("******Hello");
  });

  it("must allow to normalize texts lines width", function () {
    var text = "Hello World\nSomething\nCat";
    var expected = "Hello World\nSomething__\nCat________";
    expect(_string2.default.fixWidth(text, "_")).toEqual(expected);
  });

  it("must allow to normalize the width of strings in array", function () {
    var lines = ["Hello World", "Something", "Cat"];
    var expected = ["Hello World", "Something__", "Cat________"];
    expect(_utils2.default.equal(_string2.default.fixWidth(lines, "_"), expected)).toEqual(true);
  });

  it("must return the widths of lines in strings", function () {
    var text = "Hello World\nSomething\nCat";
    var a = _string2.default.linesWidths(text);

    expect(a instanceof Array).toEqual(true);
    expect(a[0]).toEqual(11);
    expect(a[1]).toEqual(9);
    expect(a[2]).toEqual(3);
  });

  it("must return the widths of lines in arrays", function () {
    var text = ["Hello World", "Something", "Cat"];
    var a = _string2.default.linesWidths(text);

    expect(a instanceof Array).toEqual(true);
    expect(a[0]).toEqual(11);
    expect(a[1]).toEqual(9);
    expect(a[2]).toEqual(3);
  });

  it("must allow to normalize strings for sorting and searching", function () {
    expect(_string2.default.normalize("Cześć!")).toEqual("Czesc!", "Cześć -> Czesc");
    expect(_string2.default.normalize("Łukasz")).toEqual("Lukasz", "Łukasz -> Lukasz");
  });

  it("must allow to replace strings at index", function () {
    var a = "Hello World";
    var b = _string2.default.replaceAt(a, 6, "Dear!");

    expect(b).toEqual("Hello Dear!");
  });

  it("must allow to restore diacritics in strings", function () {
    var a = "Zelazowskieasdaz";
    var diacritics = [{ i: 0, v: 'Ż' }, { i: 10, v: 'ę' }, { i: 14, v: 'ą' }, { i: 15, v: 'ż' }];

    expect(_string2.default.restoreDiacritics(a, diacritics)).toEqual("Żelazowskięasdąż");
  });

  it("must allow to restore diacritics in strings, with offset", function () {
    var a = "Henrik Nordvargr Björkk";
    var diacritics = [{ i: 19, v: "ö" }];

    var portion = "Bjorkk";
    var offset = a.indexOf("B");
    expect(_string2.default.restoreDiacritics(portion, diacritics, offset)).toEqual("Björkk");
  });

  it("must not find Chinese characters when find diacritics in strings", function () {
    var a = "2017年12月31日";
    var diacritics = _string2.default.findDiacritics(a);

    expect(diacritics).toEqual([]);
  });

  it("must allow to find diacritics in strings", function () {
    var a = "Żelazowskięasdąż";
    var diacritics = _string2.default.findDiacritics(a);

    expect(diacritics).toEqual([{ i: 0, v: 'Ż' }, { i: 10, v: 'ę' }, { i: 14, v: 'ą' }, { i: 15, v: 'ż' }]);
  });
});